@@ -11,6 +11,8 @@ from .config import settings as settings
 import PySimpleGUI as sg
 # Digi-Key API
 from .search import digikey_api as digikey_api
+# Mouser API
+from .search import mouser_api as mouser_api
 # SnapEDA API
 from .search import snapeda_api as snapeda_api
 # Progress
@@ -77,8 +79,8 @@ def user_settings_window():
     return
 
 
-def search_api_settings_window():
-    ''' Part search API settings window '''
+def digikey_api_settings_window():
+    ''' Digi-Key API settings window '''
 
     user_settings = config_interface.load_file(settings.CONFIG_DIGIKEY_API)
 
@@ -110,6 +112,7 @@ def search_api_settings_window():
             }
             user_settings = {**user_settings, **new_settings}
             config_interface.dump_file(user_settings, settings.CONFIG_DIGIKEY_API)
+            digikey_api.setup_environment(force=True)
 
         if api_event == sg.WIN_CLOSED:
             search_api_window.close()
@@ -130,6 +133,55 @@ def search_api_settings_window():
             return
 
 
+def mouser_api_settings_window():
+    ''' Mouser API settings window '''
+
+    user_settings = config_interface.load_file(settings.CONFIG_MOUSER_API)
+
+    search_api_layout = [
+        [
+            sg.Text('Mouser Part API Key '),
+            sg.InputText(user_settings['MOUSER_PART_API_KEY'], key='api_key'),
+        ],
+        [
+            sg.Button('Test', size=(15, 1)),
+            sg.Button('Save', size=(15, 1)),
+        ],
+    ]
+    search_api_window = sg.Window(
+        'Mouser API (v1) Settings', search_api_layout, location=(500, 500)
+    )
+
+    while True:
+        api_event, api_values = search_api_window.read()
+
+        def save_settings(user_settings: dict):
+            new_settings = {
+                'MOUSER_PART_API_KEY': api_values['api_key'],
+            }
+            user_settings = {**user_settings, **new_settings}
+            config_interface.dump_file(user_settings, settings.CONFIG_MOUSER_API)
+            mouser_api.setup_environment(force=True)
+
+        if api_event == sg.WIN_CLOSED:
+            search_api_window.close()
+            return
+        elif api_event == 'Test':
+            # Automatically save settings
+            save_settings(user_settings)
+            if mouser_api.test_api():
+                result_message = 'Successfully connected to Mouser API'
+            else:
+                result_message = 'Failed to connect to Mouser API'
+            sg.popup_ok(result_message,
+                        title='Mouser API Connect Test',
+                        location=(500, 500))
+        else:
+            save_settings(user_settings)
+            search_api_window.close()
+            return
+
+
 def inventree_settings_window():
     ''' InvenTree settings window '''
 
@@ -333,52 +385,21 @@ def snapeda_window(part_number: str):
     snapeda_window.close()
 
 
-def add_custom_part(part_data: dict) -> dict:
-    ''' Add custom part (bypass Digi-Key search) '''
+def part_user_form(part_data: dict) -> dict:
+    ''' User part data form '''
+
     user_values = {}
     add_custom_layout = []
 
-    def get_default(part_data, key):
-        try:
-            default_key = list(part_data[key].keys())[0]
-            default_value = part_data[key][default_key][0]
-        except:
-            default_key = ''
-            default_value = ''
-
-        return default_key, default_value
-
-    skip_items = ['category', 'IPN', 'image', 'inventree_url', 'parameters']
     input_keys = []
-    for key, value in settings.inventree_part_template.items():
-        if key in skip_items:
-            pass
-        elif key == 'supplier' or key == 'manufacturer':
-            # Get default values
-            name_default, number_default = get_default(part_data, key)
+    for key, value in part_data.items():
+        label = key.replace('_', ' ').title()
 
-            sub_key = key + '_name'
-            add_custom_layout.append([
-                sg.Text(sub_key.replace('_', ' ').title(), size=(22, 1)),
-                sg.InputText(name_default, size=(38, 1), key=sub_key),
-            ])
-            input_keys.append(sub_key)
-
-            sub_key = key + '_part_number'
-            add_custom_layout.append([
-                sg.Text(sub_key.replace('_', ' ').title(), size=(22, 1)),
-                sg.InputText(number_default, size=(38, 1), key=sub_key),
-            ])
-            input_keys.append(sub_key)
-        else:
-            default = part_data.get(key, '')
-            sub_key = key.replace('_', ' ').title()
-
-            add_custom_layout.append([
-                sg.Text(sub_key, size=(22, 1)),
-                sg.InputText(default, size=(38, 1), key=key),
-            ])
-            input_keys.append(key)
+        add_custom_layout.append([
+            sg.Text(label, size=(22, 1)),
+            sg.InputText(value, size=(38, 1), key=key),
+        ])
+        input_keys.append(key)
 
     if part_data:
         add_custom_layout.append([sg.Button('Submit', size=(25, 1)), ])
@@ -438,19 +459,16 @@ def user_defined_categories(category=None, subcategory=None, extend=False) -> li
     subcategory_default = None
 
     try:
-        for cat in categories_dict.keys():
-            categories_choices.append(cat)
+        for key in categories_dict:
+            categories_choices.append(key)
 
-            if category:
-                if category == cat:
-                    # Subcategory choices
-                    for subcat in categories_dict[cat].keys():
-                        if subcat not in subcategories_choices:
-                            subcategories_choices.append(subcat)
+            if key == category:
+                # Subcategory choices
+                subcategories_choices = [item for item in categories_dict[key]]
 
         if subcategory:
             subcategory_default = subcategory
-    except:
+    except TypeError:
         # categories_dict is None
         pass
 
@@ -570,7 +588,8 @@ def user_defined_symbol_template_footprint(categories: list,
         for cat in items.keys():
             if cat != category and cat != 'uncategorized':
                 for key in items[cat].keys():
-                    more_choices.append(key)
+                    if key not in choices and key not in more_choices:
+                        more_choices.append(key)
 
         # Process uncategorized entries
         try:
@@ -810,12 +829,23 @@ def user_defined_symbol_template_footprint(categories: list,
 
         return symbol, template, footprint
 
+
+# Init
+def init():
+    # Disable Digi-Key API logger
+    digikey_api.disable_api_logger()
+    # Fix for Windows EXE
+    import multiprocessing
+    multiprocessing.freeze_support()
+
+
 # Main
-
-
 def main():
     ''' Main GUI window '''
 
+    # Disable Digi-Key logger
+    init()
+
     CREATE_CUSTOM = False
 
     # Select PySimpleGUI theme
@@ -828,6 +858,7 @@ def main():
          [
              'User',
              'Digi-Key',
+             'Mouser',
              'KiCad',
              'InvenTree',
          ],
@@ -874,7 +905,9 @@ def main():
         if event == 'User':
             user_settings_window()
         elif event == 'Digi-Key':
-            search_api_settings_window()
+            digikey_api_settings_window()
+        elif event == 'Mouser':
+            mouser_api_settings_window()
         elif event == 'InvenTree':
             inventree_settings_window()
         elif event == 'KiCad':
@@ -892,6 +925,9 @@ def main():
             footprint = None
             new_part = False
             part_pk = 0
+            part_supplier_info = {}
+            part_supplier_form = {}
+            part_user_info = {}
             part_info = {}
             part_data = {}
             progressbar = False
@@ -922,20 +958,18 @@ def main():
 
             # Get part information
             if inventree_connect:
-                if CREATE_CUSTOM:
-                    custom_part_info = add_custom_part(part_data={})
-                    try:
-                        if custom_part_info['name'] and custom_part_info['description']:
-                            part_info = custom_part_info
-                            cprint('\n[MAIN]\tCustom Part', silent=settings.SILENT)
-                    except TypeError:
-                        pass
-                else:
-                    if values['part_number']:
-                        # New part separation
-                        new_search = '-' * 20
-                        cprint(f'\n{new_search}', silent=settings.SILENT)
+                # New part separation
+                cprint(f'\n{"-" * 20}', silent=settings.SILENT)
 
+                # Custom part
+                if CREATE_CUSTOM:
+                    cprint('\n[MAIN]\tCustom Part', silent=settings.SILENT)
+                    # Create empty user form
+                    part_supplier_form = inventree_interface.translate_supplier_to_form(supplier='custom',
+                                                                                        part_info=part_supplier_info)
+                else:
+                    # User entered part number: process with supplier search
+                    if values['part_number']:
                         # Load KiCad settings
                         settings.load_kicad_settings()
 
@@ -943,20 +977,50 @@ def main():
                         settings.load_inventree_settings()
 
                         # Supplier search
-                        part_info = inventree_interface.supplier_search(values['supplier'], values['part_number'])
+                        part_supplier_info = inventree_interface.supplier_search(values['supplier'], values['part_number'])
 
-                    if not part_info:
-                        error_message = 'Failed to fetch part information...\n\n' \
-                                        'Make sure:' \
-                                        '\n- Part number is valid and not blank'
-                        if values['supplier'] == 'Digi-Key':
-                            error_message += '\n- Digi-Key API settings are correct ("Settings > Digi-Key")'
-                        elif values['supplier'] == 'LCSC':
-                            error_message += '\n- Part number starts with "C" (LCSC code)'
-                        # Missing Part Information
-                        sg.popup_ok(error_message,
-                                    title='Supplier API Search',
-                                    location=(500, 500))
+                        if part_supplier_info:
+                            # Translate to user form format
+                            part_supplier_form = inventree_interface.translate_supplier_to_form(supplier=values['supplier'],
+                                                                                                part_info=part_supplier_info)
+                        else:
+                            error_message = 'Failed to fetch part information...\n\n' \
+                                            'Make sure:' \
+                                            '\n- Part number is valid and not blank'
+                            if values['supplier'] == 'Digi-Key':
+                                error_message += '\n- Digi-Key API settings are correct ("Settings > Digi-Key")'
+                            elif values['supplier'] == 'Mouser':
+                                error_message += '\n- Mouser API settings are correct ("Settings > Mouser")'
+                            elif values['supplier'] == 'LCSC':
+                                error_message += '\n- Part number starts with "C" (LCSC code)'
+                            # Missing Part Information
+                            sg.popup_ok(error_message,
+                                        title='Supplier API Search',
+                                        location=(500, 500))
+
+                # Do we have form data?
+                if part_supplier_form:
+                    # Open part form
+                    part_user_info = part_user_form(part_data=part_supplier_form)
+
+                    # Stitch back categories and parameters
+                    try:
+                        part_user_info.update({
+                            'category': part_supplier_info['category'],
+                            'subcategory': part_supplier_info['subcategory'],
+                            'parameters': part_supplier_info['parameters'],
+                        })
+                    except (KeyError, AttributeError):
+                        pass
+            
+            # Check that name and description are present in user form (else the form is empty)
+            try:
+                if part_user_info['name'] and part_user_info['description']:
+                    # Proceed
+                    part_info = part_user_info
+            except (KeyError, TypeError):
+                # Handle NoneType
+                pass
 
             # Get user categories
             if part_info and (settings.ENABLE_INVENTREE or settings.ENABLE_KICAD):
@@ -991,12 +1055,7 @@ def main():
             if not (categories[0] and categories[1]):
                 part_info = {}
             else:
-                if CREATE_CUSTOM:
-                    # Translate custom part data
-                    part_info = inventree_interface.translate_form_to_digikey(part_info=part_info,
-                                                                              categories=categories,
-                                                                              custom=True)
-                else:
+                if not CREATE_CUSTOM:
                     # Add to supplier categories configuration file
                     category_dict = {
                         categories[0]:
@@ -1007,23 +1066,6 @@ def main():
                         cprint(f'[INFO]\tWarning: Failed to add new supplier category to {config_file} file', silent=settings.SILENT)
                         cprint(f'[DBUG]\tcategory_dict = {category_dict}', silent=settings.SILENT)
 
-                    # Confirm part data with user
-                    form_data = add_custom_part(inventree_interface.translate_supplier_to_inventree(supplier=values['supplier'],
-                                                                                                    part_info=part_info,
-                                                                                                    categories=categories,
-                                                                                                    skip_params=True))
-                    if form_data:
-                        # Translate to part info format
-                        user_part_info = inventree_interface.translate_form_to_digikey(part_info=form_data,
-                                                                                       categories=categories,
-                                                                                       custom=False)
-
-                        # Merge original part_info with user_part_info
-                        part_info = {**part_info, **user_part_info}
-                    else:
-                        # User did not proceed
-                        part_info = {}
-
             # Set KiCad user libraries and symbol/footprint
             if part_info and settings.ENABLE_KICAD:
 
@@ -1045,8 +1087,7 @@ def main():
 
                     # Create part in InvenTree
                     if settings.ENABLE_INVENTREE:
-                        new_part, part_pk, part_data = inventree_interface.inventree_create(supplier=values['supplier'],
-                                                                                            part_info=part_info,
+                        new_part, part_pk, part_data = inventree_interface.inventree_create(part_info=part_info,
                                                                                             categories=categories,
                                                                                             kicad=settings.ENABLE_KICAD,
                                                                                             symbol=symbol,
@@ -1059,13 +1100,13 @@ def main():
                     else:
                         if not categories[0]:
                             pseudo_categories = [symbol, None]
-                            part_data = inventree_interface.translate_supplier_to_inventree(supplier=values['supplier'],
-                                                                                            part_info=part_info,
-                                                                                            categories=pseudo_categories)
+                            part_data = inventree_interface.translate_form_to_inventree(part_info=part_info,
+                                                                                        categories=pseudo_categories,
+                                                                                        is_custom=CREATE_CUSTOM)
                         else:
-                            part_data = inventree_interface.translate_supplier_to_inventree(supplier=values['supplier'],
-                                                                                            part_info=part_info,
-                                                                                            categories=categories)
+                            part_data = inventree_interface.translate_form_to_inventree(part_info=part_info,
+                                                                                        categories=categories,
+                                                                                        is_custom=CREATE_CUSTOM)
                             part_data['parameters']['Symbol'] = symbol
                             part_data['parameters']['Footprint'] = footprint
                         if not part_data:
@@ -1082,6 +1123,10 @@ def main():
                             part_data['IPN'] = part_data['name']
                     else:
                         part_data['IPN'] = values['part_number']
+
+                    # Replace spaces with hyphens
+                    part_data['IPN'] = part_data['IPN'].replace(' ', '-')
+
                     if part_data['datasheet']:
                         part_data['inventree_url'] = part_data['datasheet']
 
@@ -1198,13 +1243,3 @@ def main():
             CREATE_CUSTOM = False
 
     window.close()
-
-
-if __name__ == '__main__':
-    # Disable Digi-Key API logger
-    digikey_api.disable_api_logger()
-    # Fix for Windows EXE
-    import multiprocessing
-    multiprocessing.freeze_support()
-    # Run main window
-    main()
